{% extends "!layout.html" %}

{% block extrahead %}
<style>
  body {
    font-size: 16px;
  }

  div.topic {
    padding: 0;
    background-color: #F8F4A6;
    border: 1px solid #D8D486;
  }

  div.topic p.first {
    margin: 0;
    padding: 0.1em 0 0.1em 0.5em;
    background-color: #E8E496;
    border-bottom: 1px solid #C8C476;
  }

  div.topic p {
    margin: 0.5em 1em 0.5em 1em;
  }

  p.topic-title {
    display: run-in;
    margin-right: 15px;
  }

  a.external-link {
    background: url(_static/external.png) center right no-repeat;
    padding-right: 13px;
  }

  div.related ul li a.external-link {
    padding-right: 13px;
  }

  .sidebartext {
    font-size: 75%;
    line-height: 130%;
  }

</style>

{% if include_analytics %}
<script>
  (function(i,s,o,g,r,a,m){i['GoogleAnalyticsObject']=r;i[r]=i[r]||function(){
  (i[r].q=i[r].q||[]).push(arguments)},i[r].l=1*new Date();a=s.createElement(o),
  m=s.getElementsByTagName(o)[0];a.async=1;a.src=g;m.parentNode.insertBefore(a,m)
  })(window,document,'script','//www.google-analytics.com/analytics.js','ga');

  ga('create', 'UA-29887834-3', 'auto');
  ga('send', 'pageview');
</script>
{% endif %}

{% endblock %}


{% block rootrellink %}
  <li><a href="{{ pathto('index') }}">Home</a>&nbsp;|&nbsp;</li>
  <li><a href="{{ pathto('contents') }}">Documentation</a>&nbsp;|&nbsp;</li>
  <!-- TODO news -->
  <li><a class="external-link" href="https://github.com/pydoit/doit">Code</a>&nbsp;|&nbsp;</li>
  <li><a class="external-link" href="https://github.com/pydoit/doit/issues">Issues</a>&nbsp;|&nbsp;</li>
  <li><a class="external-link" href="http://pypi.python.org/pypi/doit">Download</a>&nbsp;|&nbsp;</li>
  <li><a class="external-link" href="http://groups.google.co.in/group/python-doit">Forum</a>&nbsp;|&nbsp;</li>
  <li><a class="external-link" href="https://twitter.com/py_doit">Twitter</a></li>
{% endblock %}


{% block header %}
<div style="background-color: white; text-align: left; padding: 10px 10px 15px 50px">
<img src="{{ pathto("_static/doit.png", 1) }}" alt="doit logo" />
</div>
{% endblock %}


{% block sidebarsearch %}

{{ super() }}

{% if include_donate %}
<h3>Funding Campaign</h3>
<iframe src="https://www.indiegogo.com/project/doit-python-automation-tool-release-0-30/embedded/2922409" width="222px" height="445px" frameborder="0" scrolling="no"></iframe>
{% endif %}

<h3>Sponsors</h3>

<div>
  <a href="http://pygarden.com"><img src="_static/pygarden.png" alt="pygarden.com logo" style="display:block; margin:auto;"/></a><br/>
</div>


<h3>Donate</h3>

{% if include_donate %}
<div>
<form action="https://www.paypal.com/cgi-bin/webscr" method="post" target="_top" style="margin:auto;">
<input type="hidden" name="cmd" value="_s-xclick">
<input type="hidden" name="hosted_button_id" value="GFU3BPVN586A6">
<input type="image" src="https://www.paypalobjects.com/en_US/i/btn/btn_donateCC_LG.gif" border="0" name="submit" alt="PayPal - The safer, easier way to pay online!">
<img alt="" border="0" src="https://www.paypalobjects.com/pt_BR/i/scr/pixel.gif" width="1" height="1">
</form>
</div>
{% endif %}
<br/>

<div class="sidebartext">
If you use <em>doit</em> and think it is a useful project. Please consider to make a donation to support the developer/maintainer with maintanance tasks (bug fixes, merge patches, reply users) and further development.<br/>
</div>


{# logo - python #}
<h3></h3>
<a href="http://python.org"><img src="_static/python-powered-w-100x40.png" alt="python logo"/></a><br/><br/>
{% endblock %}

{% block sidebarrel %}
{% endblock %}

